

//    ________________
//   / External DLL   \
// ------------------------------------------------------
//
// This DLL can be used to transfer upto 6 variables from an external source into Synfactory.
// The values will arive as JoyStick coordinates and therefore must lay between
// -32768 and 32767 (inclusive), BUT because of our friends at redmond this is encoded as
// 0 to 65535 so add 32768 to the value before writing to the structure!
//
// 4 Functions are used to interface between this DLL and SynFactory and the prototypes
// must match EXACTLY to prevent unnecessary crash drama:
//
// char * WINAPI external_getversion(void);
// int WINAPI external_getvalues(JOYINFOEX *joystickValues);
// void WINAPI external_initdll(HWND application, logprintfFunc applogprintf);
// void WINAPI external_deinitdll(void);
//
// 'application' is a pointer to the SynFactory main window HWND (use with care!)
// logprintf has same syntax as printf/fprintf/sprintf functions and can be used to log
// information to the synfactory logfile.
//
// DLL is loaded dynamically. If DLL with the name sf_external.dll is not found or one
// of the 4 functions above is are found, the DLL is not loaded and SynFactory will use the
// normal joystick as input.
// To use the values inside synfactory create a JOYST module. Use X,Y,Z,R,U and V parameters
// set in the structure from this DLL. (The 4 button states are not read)
// In menu Config/Setting set joystick dropdown box on <<no joystick control>> otherwise
// the real joystick is used instead of this DLL!!!
//
// Use the provided logprintf for debugging. This routine is multi-thread save.
// Don't know the exact flow when more than 1 instance of synfactory is started, be wise
// and don't do that :-)
//
//
// Rebuild instructions:
//
// Create workspace in VC++6 (atleast that is what I used) for a dynamicDLL. Respond to wizard
// with empty project with no files. Add sf_external.cpp and sf_external.def to project and press
// F7 wahalla!! a dll is generated. Put into synfactory directory and have fun!
// This example increases X with high speed so can be used to test interface. After verified this
// is working add own code at the places marked with comment statements.


#pragma comment(linker,"/opt:nowin98 /MAP /MAPINFO:FIXUPS")  /* VC6: turn off 4kbyte alignment. */

#include <windows.h>

typedef int (__cdecl *logprintfFunc)(const char *formatstr, ...);

static HANDLE myInstance;
static HWND applicationHWND;
static logprintfFunc logprintf=NULL;


char * WINAPI external_getversion(void) {
  return "SF_EXTERNAL.DLL EXAMPLE V1.00";
}


//
// Called around 20 times a second when audioplayback is active.
// return 0 (JOYERR_NOERROR) when values are written.
// return !=0 when error. (structure contents will be ignored by synfactory)
//
int WINAPI external_getvalues(JOYINFOEX *joystickValues) {
  static int t=0;

  /*-----------------------------------------*/
  /* Insert database read logic here         */
  /*-----------------------------------------*/

  joystickValues->dwXpos=t;      /* Value 1 */
  joystickValues->dwYpos=32768;  /* Value 2 */
  joystickValues->dwZpos=32768;  /* Value 3 */
  joystickValues->dwRpos=32768;  /* Value 4 */
  joystickValues->dwUpos=32768;  /* Value 5 */
  joystickValues->dwVpos=32768;  /* Value 6 */

  t=t+384;
  if (t>65535) t=0;

  return JOYERR_NOERROR;
}



//
// Called during startup of SynFactory
//
void WINAPI external_initdll(HWND application, logprintfFunc applogprintf) {
  applicationHWND=application;
  logprintf=applogprintf;
  
  /*-----------------------------------------*/
  /* Insert initialisation functions here    */
  /*-----------------------------------------*/

  logprintf("External DLL installed and ready.\n");
}


//
// Called during shutdown of SynFactory
//
void WINAPI external_deinitdll(void) {

  /*-----------------------------------------------*/
  /* Insert shutdown and release functions here    */
  /*-----------------------------------------------*/

  logprintf("External DLL shutdown complete.\n");
}





BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved) {

  switch (ul_reason_for_call) {
	case DLL_PROCESS_ATTACH:
    myInstance=hModule;
    break;
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
  }

  return TRUE;
}
